'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var BRACKETS = ['(', ')', '{', '}'];

function lex(_ref) {
  var quasis = _ref.quasis;
  var expressions = _ref.expressions;

  var tokens = [];
  quasis.forEach(function (quasi, i) {
    var str = quasi.value.raw;
    // TODO: track line number within quasi
    var line = quasi.loc.start.line;
    function trim() {
      while (str !== str.trim() || str[0] === ',') {
        str = str.trim();
        if (str[0] === ',') str = str.substr(1);
      }
    }
    function lexKeyword() {
      var match = /^(if|else|as)\b/.exec(str);
      if (match) {
        str = str.substr(match[0].length);
        return { type: 'Keyword', val: match[0], line: line };
      }
      return null;
    }
    function lexString() {
      if (str[0] === '"' || str[0] === '\'') {
        var quote = str[0];
        var val = '';
        var escaped = false;
        var _i = 1;
        while (_i < str.length && (escaped || str[_i] !== quote)) {
          if (str[_i] === '\\') escaped = true;
          if (str[_i] === '"' && !escaped) val += '\\';
          val += str[_i++];
        }
        if (str[_i] === quote) {
          str = str.substr(_i + 1);
          return { type: 'String', val: JSON.parse('"' + val + '"'), line: line };
        } else {
          throw new Error('Missing closing quote');
        }
      }
      return null;
    }
    function lexBool() {
      var match = /^(true|false)\b/.exec(str);
      if (match) {
        str = str.substr(match[0].length);
        return { type: 'Bool', val: match[0] === 'true', line: line };
      }
      return null;
    }
    function lexNumber() {
      var match = /^\-?\d+(\.\d+)?/.exec(str);
      if (match) {
        str = str.substr(match[0].length);
        return { type: 'Number', val: +match[0], line: line };
      }
      return null;
    }
    function lexBracket() {
      if (BRACKETS.indexOf(str[0]) !== -1) {
        var match = str[0];
        str = str.substr(1);
        return { type: 'Bracket', val: match, line: line };
      }
      return null;
    }
    function lexColon() {
      if (str[0] === ':') {
        str = str.substr(1);
        return { type: 'Colon', line: line };
      }
      return null;
    }
    function lexSpread() {
      if (str[0] === '.' && str[1] === '.' && str[2] === '.') {
        str = str.substr(3);
        return { type: 'Spread', line: line };
      }
      return null;
    }
    function lexIdentifier() {
      var match = /^[a-zA-Z]+\b/.exec(str);
      if (match) {
        str = str.substr(match[0].length);
        return { type: 'Identifier', val: match[0], line: line };
      }
      return null;
    }
    function lexComment() {
      if (str[0] === '/' && str[1] === '/') {
        str = str.substr(str.indexOf('\n'));
        return true;
      }
      return false;
    }
    function fail() {
      throw new Error('Unable to lex at "' + str + '"');
    }
    trim();
    while (str.length) {
      if (!lexComment()) {
        tokens.push(lexKeyword() || lexString() || lexBool() || lexNumber() || lexBracket() || lexColon() || lexSpread() || lexIdentifier() || fail());
      }
      trim();
    }
    if (expressions[i]) {
      tokens.push({ type: 'Expression', val: expressions[i] });
    }
  });
  tokens.push({ type: 'End' });
  return tokens;
}
exports.default = lex;