'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _tokenStream = require('token-stream');

var _tokenStream2 = _interopRequireDefault(_tokenStream);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parse(tokens) {
  tokens = new _tokenStream2.default(tokens);
  function parseArg() {
    var identifier = tokens.advance();
    (0, _assert2.default)(identifier.type === 'Identifier');

    var colon = tokens.advance();
    (0, _assert2.default)(colon.type === 'Colon');

    var val = tokens.advance();

    return { type: 'Arg', name: identifier.val, val: val };
  }
  function parseArgs() {
    var openBracket = tokens.advance();
    (0, _assert2.default)(openBracket.type === 'Bracket' && openBracket.val === '(');
    var args = [];
    while (tokens.peek().type !== 'Bracket') {
      args.push(parseArg());
    }
    var closeBracket = tokens.advance();
    (0, _assert2.default)(closeBracket.type === 'Bracket' && closeBracket.val === ')');
    return args;
  }
  function parseFieldSet() {
    var openBracket = tokens.advance();
    (0, _assert2.default)(openBracket.type === 'Bracket' && openBracket.val === '{');
    var body = parseBody();
    var closeBracket = tokens.advance();
    (0, _assert2.default)(closeBracket.type === 'Bracket' && closeBracket.val === '}');
    return { type: 'FieldSet', body: body };
  }
  function parseAlias() {
    var keyword = tokens.advance();
    (0, _assert2.default)(keyword.type === 'Keyword' && keyword.val === 'as');
    var identifier = tokens.advance();
    (0, _assert2.default)(identifier.type === 'Identifier');
    return identifier.val;
  }
  function parseField() {
    var identifier = tokens.advance();
    (0, _assert2.default)(identifier.type === 'Identifier');
    var args = null;
    var body = null;
    var alias = null;
    if (tokens.peek().type === 'Bracket' && tokens.peek().val === '(') {
      args = parseArgs();
    }
    if (tokens.peek().type === 'Keyword' && tokens.peek().val === 'as') {
      alias = parseAlias();
      if (alias === identifier.val) alias = null;
    }
    if (tokens.peek().type === 'Bracket' && tokens.peek().val === '{') {
      body = parseFieldSet();
    }
    return { type: 'Field', name: identifier.val, args: args, alias: alias, body: body };
  }
  function parseSpread() {
    var spread = tokens.advance();
    (0, _assert2.default)(spread.type === 'Spread');
    var arg = tokens.advance();
    (0, _assert2.default)(arg.type === 'Expression');
    return { type: 'Spread', arg: arg };
  }
  function parseKeyword() {
    var keyword = tokens.advance();
    (0, _assert2.default)(keyword.type === 'Keyword');
    (0, _assert2.default)(keyword.val === 'if');
    var condition = tokens.advance();
    (0, _assert2.default)(condition.type === 'Expression');
    var consequent = parseFieldSet();
    var alternate = null;
    if (tokens.peek().type !== 'Keyword' && tokens.peek().val === 'else') {
      tokens.advance();
      alternate = parseFieldSet();
    }
    return { type: 'Conditional', condition: condition, consequent: consequent, alternate: alternate };
  }
  function parseBody() {
    var body = [];
    while ((tokens.peek().type !== 'Bracket' || tokens.peek().val !== '}') && tokens.peek().type !== 'End') {
      switch (tokens.peek().type) {
        case 'Identifier':
          body.push(parseField());
          break;
        case 'Spread':
          body.push(parseSpread());
          break;
        case 'Keyword':
          body.push(parseKeyword());
          break;
        /*
        case 'Keyword':
          body.push(parseKeyword());
          break;
        */
        default:
          throw new Error('Unexpected token ' + tokens.peek().type + '\n' + (0, _stringify2.default)(tokens.peek(), null, '  '));
      }
    }
    return body;
  }
  return { type: 'FieldSet', body: parseBody() };
}
exports.default = parse;